@@ -152,8 +152,11 @@ gail_get_accessible_for_widget (GtkWidget *widget,
   else if (GTK_CHECK_TYPE ((widget), gnome_canvas))
     {
       GObject *focused_item;
+      GValue value = {0, };
 
-      focused_item = g_object_get_data (G_OBJECT (widget), "focused_item");
+      g_value_init (&value, G_TYPE_OBJECT);
+      g_object_get_property (G_OBJECT (widget), "focused_item", &value);
+      focused_item = g_value_get_object (&value);
 
       if (focused_item)
         {
