@@ -80,7 +80,7 @@ static AtkAttributeSet* gail_item_get_run_attributes
 					            gint	      *end_offset);
 static AtkAttributeSet* gail_item_get_default_attributes
                                                    (AtkText           *text);
-static GtkWidget*            get_label_from_item   (GtkWidget         *item);
+static GtkWidget*            get_label_from_container   (GtkWidget    *container);
 
 
 
@@ -174,8 +174,9 @@ gail_item_real_initialize (AtkObject *obj,
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
   item->textutil = NULL;
+  item->text = NULL;
 
-  label = get_label_from_item (GTK_WIDGET (data));
+  label = get_label_from_container (GTK_WIDGET (data));
   if (GTK_IS_LABEL (label))
     {
       if (GTK_WIDGET_MAPPED (label))
@@ -227,6 +228,11 @@ gail_item_finalize (GObject *object)
     {
       g_object_unref (item->textutil);
     }
+  if (item->text)
+    {
+      g_free (item->text);
+      item->text = NULL;
+    }
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -238,9 +244,7 @@ gail_item_get_name (AtkObject *obj)
   g_return_val_if_fail (GAIL_IS_ITEM (obj), NULL);
 
   name = ATK_OBJECT_CLASS (parent_class)->get_name (obj);
-  if (name != NULL)
-    return name;
-  else
+  if (name == NULL)
     {
       /*
        * Get the label child
@@ -255,7 +259,7 @@ gail_item_get_name (AtkObject *obj)
          */
         return NULL;
 
-      label = get_label_from_item (widget);
+      label = get_label_from_container (widget);
       if (GTK_IS_LABEL (label))
 	return gtk_label_get_text (GTK_LABEL(label));
       /*
@@ -263,26 +267,71 @@ gail_item_get_name (AtkObject *obj)
        * the label of the selected item is detached from the menu item
        */
       else if (GTK_IS_MENU_ITEM (widget))
-         {
-           GtkWidget *parent;
-           GtkWidget *attach;
+        {
+          GtkWidget *parent;
+          GtkWidget *attach;
+          GList *list;
+          AtkObject *parent_obj;
+          gint index;
 
-           parent = gtk_widget_get_parent (widget);
-           if (GTK_IS_MENU (parent))
-             {
-               attach = gtk_menu_get_attach_widget (GTK_MENU (parent)); 
+          parent = gtk_widget_get_parent (widget);
+          if (GTK_IS_MENU (parent))
+            {
+              attach = gtk_menu_get_attach_widget (GTK_MENU (parent)); 
 
-               if (GTK_IS_OPTION_MENU (attach))
-                 {
-                   label = get_label_from_item (attach);
-                   if (GTK_IS_LABEL (label))
-	             return gtk_label_get_text (GTK_LABEL(label));
-                 }
-             }
-         }
+              if (GTK_IS_OPTION_MENU (attach))
+                {
+                  label = get_label_from_container (attach);
+                  if (GTK_IS_LABEL (label))
+	            return gtk_label_get_text (GTK_LABEL(label));
+                }
+              list = gtk_container_get_children (GTK_CONTAINER (parent));
+              index = g_list_index (list, widget);
 
-      return NULL;
+              if (index < 0 || index > g_list_length (list))
+                {
+                  g_list_free (list);
+                  return NULL;
+                }
+              g_list_free (list);
+
+              parent_obj = atk_object_get_parent (gtk_widget_get_accessible (parent));
+              if (GTK_IS_ACCESSIBLE (parent_obj))
+                {
+                  parent = GTK_ACCESSIBLE (parent_obj)->widget;
+                  if (GTK_IS_COMBO_BOX (parent))
+                    {
+                      GtkTreeModel *model;
+                      GtkTreeIter iter;
+                      GailItem *item;
+                      gint n_columns, i;
+
+                      model = gtk_combo_box_get_model (GTK_COMBO_BOX (parent));                       
+                      item = GAIL_ITEM (obj);
+                      if (gtk_tree_model_iter_nth_child (model, &iter, NULL, index))
+                        {
+                          n_columns = gtk_tree_model_get_n_columns (model);
+                          for (i = 0; i < n_columns; i++)
+                            {
+                              GValue value = { 0, };
+
+                               gtk_tree_model_get_value (model, &iter, i, &value);
+                               if (G_VALUE_HOLDS_STRING (&value))
+                                 {
+				   g_free (item->text);
+                                   item->text =  (gchar *) g_value_dup_string (&value);
+                                   g_value_unset (&value);
+                                   break;
+                                 }
+                            }
+                        }
+                      name = item->text;
+                    }
+                }
+            }
+        }
     }
+  return name;
 }
 
 /*
@@ -369,7 +418,7 @@ gail_item_get_text (AtkText *text,
     /* State is defunct */
     return NULL;
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL (label))
     return NULL;
@@ -407,7 +456,7 @@ gail_item_get_text_before_offset (AtkText         *text,
     return NULL;
   
   /* Get label */
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return NULL;
@@ -439,7 +488,7 @@ gail_item_get_text_at_offset (AtkText         *text,
     return NULL;
   
   /* Get label */
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return NULL;
@@ -473,7 +522,7 @@ gail_item_get_text_after_offset (AtkText         *text,
   }
   
   /* Get label */
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return NULL;
@@ -498,7 +547,7 @@ gail_item_get_character_count (AtkText *text)
     /* State is defunct */
     return 0;
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return 0;
@@ -527,7 +576,7 @@ gail_item_get_character_extents (AtkText      *text,
     /* State is defunct */
     return;
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return;
@@ -557,7 +606,7 @@ gail_item_get_offset_at_point (AtkText      *text,
     /* State is defunct */
     return -1;
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return -1;
@@ -596,7 +645,7 @@ gail_item_get_run_attributes (AtkText *text,
     /* State is defunct */
     return NULL;
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return NULL;
@@ -638,7 +687,7 @@ gail_item_get_default_attributes (AtkText *text)
     /* State is defunct */
     return NULL;
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return NULL;
@@ -663,7 +712,7 @@ gail_item_get_character_at_offset (AtkText *text,
     /* State is defunct */
     return '\0';
 
-  label = get_label_from_item (widget);
+  label = get_label_from_container (widget);
 
   if (!GTK_IS_LABEL(label))
     return '\0';
@@ -676,15 +725,15 @@ gail_item_get_character_at_offset (AtkText *text,
 }
 
 static GtkWidget*
-get_label_from_item (GtkWidget *item)
+get_label_from_container (GtkWidget *container)
 {
   GtkWidget *label;
   GList *children, *tmp_list;
 
-  if (!GTK_IS_CONTAINER (item))
+  if (!GTK_IS_CONTAINER (container))
     return NULL;
  
-  children = gtk_container_get_children (GTK_CONTAINER (item));
+  children = gtk_container_get_children (GTK_CONTAINER (container));
   label = NULL;
 
   for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
@@ -694,6 +743,16 @@ get_label_from_item (GtkWidget *item)
            label = tmp_list->data;
            break;
 	}
+      /*
+       * Get label from menu item in desktop background preferences
+       * option menu. See bug #144084.
+       */
+      else if (GTK_IS_BOX (tmp_list->data))
+        {
+           label = get_label_from_container (GTK_WIDGET (tmp_list->data));
+           if (label)
+             break;
+        }
     }
   g_list_free (children);
 
